/*
**    TP 4ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <z_buffer.hpp>

#include <p2d.hpp>
#include <exception_cpe.hpp>
#include <image_io.hpp>

#include <sstream>

namespace cpe
{
z_buffer::z_buffer()
    :internal_size_x(0),internal_size_y(0){}
z_buffer::z_buffer(const unsigned int& size)
    :internal_data(size*size),internal_size_x(size),internal_size_y(size)
{
    double infinity=depth_infinity;
    fill(infinity);
}

z_buffer::z_buffer(const unsigned int& size_x,const unsigned int& size_y)
    :internal_data(size_x*size_y),internal_size_x(size_x),internal_size_y(size_y)
{
    double infinity=depth_infinity;
    fill(infinity);
}

void z_buffer::fill(const double& value)
{
    std::fill(internal_data.begin(),internal_data.end(),value);
}

const unsigned int& z_buffer::size_x() const{return internal_size_x;}
const unsigned int& z_buffer::size_y() const{return internal_size_y;}
const double& z_buffer::operator()(const p2d& u) const
{
    check_index(u);

    int index=coordinate_to_index(u);
    return internal_data[index];
}
double& z_buffer::operator()(const p2d& u)
{
    check_index(u);

    int index=coordinate_to_index(u);
    return internal_data[index];
}

int z_buffer::coordinate_to_index(const p2d& u) const
{
    return u.x()+internal_size_x*u.y();
}
void z_buffer::check_index(const p2d& u) const
{
    if(u.x()<0 || u.x()>=static_cast<int>(internal_size_x) || u.y()<0 || u.y()>=static_cast<int>(internal_size_y))
    {
        std::ostringstream msg;
        msg<<"Error try to access ["<<u<<"], while size is ["<<internal_size_x<<","<<internal_size_y<<"]";
        throw exception_cpe(msg.str(),__FILE__,__PRETTY_FUNCTION__,__LINE__);
    }
}

void z_buffer::export_file(const std::string& filename) const
{
    image_io::export_ppm(filename,*this);
}

double z_buffer::get_max() const
{
    double current_max=0;
    for(unsigned int k=0,N=internal_data.size();k<N;++k)
    {
        const double& v=internal_data[k];
        if( v>current_max && v<depth_infinity-10.0)
            current_max=v;
    }

    return current_max;
}
double z_buffer::get_min() const
{
    double current_min=depth_infinity;
    for(unsigned int k=0,N=internal_data.size();k<N;++k)
    {
        const double& v=internal_data[k];
        if(v<current_min)
            current_min=v;
    }

    return current_min;
}

}
